// 
// fpg_tb.sv, by Per Christian Corneliussen, March 2011.
//
// SystemVerilog test bench for fpg.v. This is an informal test bench, it
// simply tests a number of Mandelbrot points that was calculated beforehand
// with the software implementation.
//

module fpg_tb();

  // DUT ports:
  logic clk, reset_n, enable, busy;
  logic [31:0] c_re, c_im;
  logic [7:0] n;
  logic [15:0] addr_in, addr_out;

  // Connect DUT ports:
  fpg dut(.*);

  typedef struct packed {
    bit [31:0] re;
    bit [31:0] im;
    bit [7:0] n;
  } test_vector_t;

  // Test vectors:
  test_vector_t test_vector [] = '{
    {32'h40000000, 32'h40000000, 8'd1},
    {32'h3c23d70a, 32'h3f851eb0, 8'd6},
    {32'h3ba3d70a, 32'h3f80a3d7, 8'd8},
    {32'h3dcccccd, 32'h3f266666, 8'd14},
    {32'h3dcccccd, 32'h3f23d70a, 8'd48},
    {32'h3dcccccd, 32'h3f1aca58, 8'd65},
    {32'h3e0ea4a9, 32'h3f1aca9b, 8'd78},
    {32'h0, 32'h0, 8'd80}
  };

  assign addr_in = 16'b0;

  // Clock driver:
  always #5 clk = !clk;

  clocking cc @(posedge clk);
    default input #1 output negedge;
    output reset_n;
    output enable, c_re, c_im;
    input busy, n;
  endclocking

  initial begin
    // Enable dumping:
    $vcdpluson(0, dut);

    // Initiate clock and reset DUT:
    clk = 0;
    reset_n <= 0;
    enable <= 0;
    
    ##1 cc.reset_n <= 1; // Will be asserted on the negative edge.
  
    foreach(test_vector[i]) begin
      // Send test vector to DUT:
      cc.c_re <= test_vector[i].re;
      cc.c_im <= test_vector[i].im;
      cc.enable <= 1'b1;
      ##1 cc.enable <= 1'b0;
      // Wait for valid-signal:
      @(negedge cc.busy);
      // Assert answer:
      assert (cc.n == test_vector[i].n) $display("PASS %d", i);
      else $error("FAIL %d (Expected %d, got %d)", i, test_vector[i].n, cc.n);
    end
    $finish;
  end
endmodule
