//
// vag_tb.v, by Per Christian Corneliussen, May 2011
// 
//SystemVerilog test bench for vag.v. As fpg_tb.sv, this is an informal test
// bench that simply requests a single landscape from the vertex array generator
// and checks the result with a solution calculated earlier with the software
// implementation.
//

module vag_tb();

  // DUT ports:
  logic clk, reset_n, start, next, ready;
  logic [31:0] left, bottom, step;
  logic [47:0] arrout;

  // Connect DUT ports:
  vag dut(.*);

  // Test data:
  logic [47:0] vertices [0:759]; // 19 strips * 40 vertices = 760
  assign left = 32'hbefb870e;
  assign bottom = 32'h3f141b79;
  assign step = 32'h3c2c7692;
  assign addr_in = 16'b0;

  // Loop counters:
  integer i, j;

  // Clock cycle counter:
  integer ccs;
  always @ (posedge clk or negedge reset_n)
    if (!reset_n) ccs <= 0;
    else ccs <= ccs + 1;

  // Clock driver:
  always #5 clk = !clk;

  default clocking cc @(posedge clk);
    default input #1 output negedge;
    output reset_n;
    output start, next;
    input ready, arrout;
  endclocking

  initial begin
    // Enable dumping:
    $vcdpluson(0, dut);

    // Load golden solution from testcase.txt:
    $readmemh("testcase.txt", vertices);

    // Initiate clock and reset DUT:
    clk = 0;
    reset_n <= 0;
    start <= 0;
    next <= 0;

    ##1 cc.reset_n <= 1;

    for (i=0; i < 19; i=i+1) begin
      // For each strip.
      $display("Requesting strip %d.", i);

      if (i == 0) begin
        cc.start <= 1;
        ##1 cc.start <= 0;
      end else begin
        cc.next <= 1;
        ##1 cc.next <= 0;
      end
      // Wait for ready-signal.
      $display("Waiting for ready-signal...");
      @(posedge cc.ready);
      // Assert answer:
      for (j=0; j < 40; j++) begin
        assert (cc.arrout == vertices[i * 40 + j]) $display("PASS %d,%d", i, j);
        else $error("FAIL %d,%d (Expected %h, got %h)", i, j,
          vertices[i * 40 + j], cc.arrout);
        ##1; // Wait one clock cycle.
      end
    end
    $display("Test bench completed after %d clock cycles.", ccs);
    $finish;
  end
endmodule

