//
// vag_tb2.v, by Per Christian Corneliussen, June 2011
// 
//SystemVerilog test bench for vag.v. This test bench will test a 400 by 400
// fractal landscape consisting of black points only. The point is to measure
// performance.
//

module vag_tb();

  // DUT ports:
  logic clk, reset_n, start, next, ready;
  logic [31:0] left, bottom, step;
  logic [47:0] arrout;

  // Connect DUT ports:
  vag dut(.*);

  // Test data:
  assign left = 32'hbe67295c;
  assign bottom = 32'hbe67295c;
  assign step = 32'h3a94506e;
  assign addr_in = 16'b0;

  // Loop counters:
  integer i, j;

  // Clock cycle counter:
  integer ccs;
  always @ (posedge clk or negedge reset_n)
    if (!reset_n) ccs <= 0;
    else ccs <= ccs + 1;

  // Clock driver:
  always #5 clk = !clk;

  default clocking cc @(posedge clk);
    default input #1 output negedge;
    output reset_n;
    output start, next;
    input ready, arrout;
  endclocking

  initial begin
    // Initiate clock and reset DUT:
    clk = 0;
    reset_n <= 0;
    start <= 0;
    next <= 0;

    ##1 cc.reset_n <= 1;

    for (i=0; i < 399; i=i+1) begin
      // For each strip.
      $display("Requesting strip %d.", i);

      if (i == 0) begin
        cc.start <= 1;
        ##1 cc.start <= 0;
      end else begin
        cc.next <= 1;
        ##1 cc.next <= 0;
      end
      // Wait for ready-signal.
      $display("Waiting for ready-signal...");
      @(posedge cc.ready);
      // Assert answer:
      for (j=0; j < 800; j++) begin
        assert (cc.arrout[15:0] == 80) $display("PASS %d,%d", i, j);
        else $error("FAIL %d,%d (Expected 80, got %h)", i, j,
          cc.arrout[15:0]);
        ##1; // Wait one clock cycle.
      end
    end
    $display("Test bench completed after %d clock cycles.", ccs);
    $finish;
  end
endmodule

